/*
 * Copyright 2022 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WRUNGEKUTTA_H
#define WRUNGEKUTTA_H

#include "waves.h"
#include "wTimeIntegration.h"
#include <Eigen/Sparse>
#include <vector>
#include <iostream>

namespace waves
{

/**
 * @brief explicit Runge Kutta time integration
 */

class WAVES_API RungeKutta : public TimeIntegration
{
public:
    RungeKutta(std::shared_ptr<Problem> _pbl);
    virtual ~RungeKutta() { std::cout << "~RungeKutta()\n"; }
    virtual char const *clsname() const override { return "RungeKutta"; }

    virtual void start(tbox::MshExport *mshWriter) override;

private:
    void predictor(std::vector<double> &up2, std::vector<double> &vp2, // premier membre
                   std::vector<double> &up1, std::vector<double> &vp1, // predicteur
                   std::vector<double> &u0, std::vector<double> &v0,   // sol au temps precedent
                   double dt,
                   std::vector<double> &Md,
                   Eigen::SparseMatrix<double, Eigen::RowMajor> &K,
                   Eigen::SparseMatrix<double, Eigen::RowMajor> &S, double ak, double t);
};

} // namespace waves

#endif //WRUNGEKUTTA_H
